package net.lemnik.eodsql;

import java.lang.reflect.Method;
import java.lang.reflect.Type;

/**
 * This method is thrown by the {@link QueryTool} class when a data-type
 * returned by an {@link Select @Select} method is found to be invalid.
 * @author jason
 */
public class InvalidDataTypeException extends InvalidQueryException {
    private Type type;

    /**
     * Creates a new instance of <code>InvalidDataTypeException</code>
     * without detail message.
     * @param type the offending data-type
     */
    public InvalidDataTypeException(Class type) {
        super("dataType=" + type);
        this.type = type;
    }
    
    /**
     * Constructs an instance of <code>InvalidDataTypeException</code>
     * with the specified detail message.
     * @param type the offending data-type
     * @param msg the detail message.
     */
    public InvalidDataTypeException(String msg, Type type) {
        super(msg + " [dataType=" + type + "]");
        this.type = type;
    }

    /**
     * Constructs an instance of <code>InvalidDataTypeException</code>
     * with the specified detail message.
     * @param type the offending data-type
     * @param msg the detail message.
     */
    public InvalidDataTypeException(String msg, Class type) {
        super(msg + " [dataType=" + type + "]");
        this.type = type;
    }

    /**
     * Constructs an instance of <code>InvalidDataTypeException</code>
     * with the specified detail message, and a specific method that
     * caused the problem.
     * @param type the offending data-type
     * @param problem the method that caused the exception to be thrown
     * @param msg the detail message.
     */
    public InvalidDataTypeException(String msg, Class type, Method problem) {
        super(msg + " [dataType=" + type + "]", problem);
        this.type = type;
    }

    /**
     * Returns the <code>Class</code> of the data-type that caused this exception
     * to be thrown. If the offending type was not a {@code Class}, this method will
     * return {@literal null}.
     * 
     * @return the offending data-type
     */
    public Class getDataType() {
        return type instanceof Class ? (Class)type : null;
    }
    
    /**
     * Returns the full data-type that  caused this exception
     * to be thrown.
     * 
     * @return the offending data-type
     */
    public Type getGenericDataType() {
        return type;
    }

}
